<?php

namespace Bixo\Schedule\Http\Controllers;

use Exception;
use Bixo\Schedule\Actions\AttendeeWorkflow;
use Bixo\Schedule\Http\Requests\AttendeeWorkflowRequest;
use Bixo\Schedule\Http\Resources\AttendeeResource;
use Bixo\Schedule\Models\Attendee;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AttendeeWorkflowController extends BaseController
{
    /**
     * Action controller function for attendee.
     *
     * @param Model $attendee
     * @param action next action for the attendee.
     *
     * @return Response
     */
    public function __invoke(AttendeeWorkflowRequest $request, Attendee $attendee, $trasition)
    {
        try {
            $request = $request->all();
            $attendee = AttendeeWorkflow::run($trasition, $attendee, $request);
            $data = new AttendeeResource($attendee);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('schedule::attendee.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('schedule/attendee/' . $attendee->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('schedule/attendee/' . $attendee->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the attendee and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $attendee
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $attendee, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $attendee = Attendee::findBySignedId($attendee);
        $transitions = $this->transitions($attendee, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('schedule::attendee.name') . ' ' . trans('Approvals'))
            ->view('schedule::public.attendee.workflow')
            ->layout('mobile')
            ->data(compact('attendee', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the attendee.
     *
     * @param PublicRequest $request
     * @param string $attendee
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $attendee, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $attendee = Attendee::findBySignedId($attendee);
        $user = user();
        $attendee = AttendeeWorkflow::run($transition, $attendee, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($attendee, $user)
    {
        $transitions = [];
        foreach ($attendee->workflow()->transitions($attendee) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.attendee.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'attendee' => $attendee->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $attendee->workflow()->form($value);
            $array['label'] = trans('schedule::attendee.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
