<?php

namespace Bixo\Schedule\Http\Controllers;

use Exception;
use Bixo\Schedule\Actions\CalendarWorkflow;
use Bixo\Schedule\Http\Requests\CalendarWorkflowRequest;
use Bixo\Schedule\Http\Resources\CalendarResource;
use Bixo\Schedule\Models\Calendar;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class CalendarWorkflowController extends BaseController
{
    /**
     * Action controller function for calendar.
     *
     * @param Model $calendar
     * @param action next action for the calendar.
     *
     * @return Response
     */
    public function __invoke(CalendarWorkflowRequest $request, Calendar $calendar, $trasition)
    {
        try {
            $request = $request->all();
            $calendar = CalendarWorkflow::run($trasition, $calendar, $request);
            $data = new CalendarResource($calendar);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('schedule::calendar.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('schedule/calendar/' . $calendar->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('schedule/calendar/' . $calendar->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the calendar and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $calendar
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $calendar, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $calendar = Calendar::findBySignedId($calendar);
        $transitions = $this->transitions($calendar, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('schedule::calendar.name') . ' ' . trans('Approvals'))
            ->view('schedule::public.calendar.workflow')
            ->layout('mobile')
            ->data(compact('calendar', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the calendar.
     *
     * @param PublicRequest $request
     * @param string $calendar
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $calendar, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $calendar = Calendar::findBySignedId($calendar);
        $user = user();
        $calendar = CalendarWorkflow::run($transition, $calendar, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($calendar, $user)
    {
        $transitions = [];
        foreach ($calendar->workflow()->transitions($calendar) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.calendar.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'calendar' => $calendar->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $calendar->workflow()->form($value);
            $array['label'] = trans('schedule::calendar.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
