<?php

namespace Bixo\Schedule\Http\Controllers;

use Exception;
use Bixo\Schedule\Actions\TaskWorkflow;
use Bixo\Schedule\Http\Requests\TaskWorkflowRequest;
use Bixo\Schedule\Http\Resources\TaskResource;
use Bixo\Schedule\Models\Task;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class TaskWorkflowController extends BaseController
{
    /**
     * Action controller function for task.
     *
     * @param Model $task
     * @param action next action for the task.
     *
     * @return Response
     */
    public function __invoke(TaskWorkflowRequest $request, Task $task, $trasition)
    {
        try {
            $request = $request->all();
            $task = TaskWorkflow::run($trasition, $task, $request);
            $data = new TaskResource($task);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('schedule::task.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('schedule/task/' . $task->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('schedule/task/' . $task->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the task and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $task
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $task, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $task = Task::findBySignedId($task);
        $transitions = $this->transitions($task, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('schedule::task.name') . ' ' . trans('Approvals'))
            ->view('schedule::public.task.workflow')
            ->layout('mobile')
            ->data(compact('task', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the task.
     *
     * @param PublicRequest $request
     * @param string $task
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $task, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $task = Task::findBySignedId($task);
        $user = user();
        $task = TaskWorkflow::run($transition, $task, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($task, $user)
    {
        $transitions = [];
        foreach ($task->workflow()->transitions($task) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.task.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'task' => $task->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $task->workflow()->form($value);
            $array['label'] = trans('schedule::task.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
